package com.usky.iot.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.aliyun.dingtalkoauth2_1_0.models.GetAccessTokenResponse;
import com.aliyun.tea.TeaException;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.dingtalk.api.DefaultDingTalkClient;
import com.dingtalk.api.DingTalkClient;
import com.dingtalk.api.request.OapiReportCreateRequest;
import com.dingtalk.api.request.OapiReportSavecontentRequest;
import com.dingtalk.api.request.OapiV2UserGetbymobileRequest;
import com.dingtalk.api.response.OapiReportCreateResponse;
import com.dingtalk.api.response.OapiV2UserGetbymobileResponse;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.taobao.api.ApiException;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.iot.constant.dingTalkConstant;
import com.usky.iot.domain.*;
import com.usky.iot.mapper.*;
import com.usky.iot.service.MceMbuserService;
import com.usky.iot.service.PmProjectService;
import com.usky.iot.service.PmWorkContentService;
import com.usky.iot.service.PmWorkReportService;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.iot.service.vo.PmProjectWorkTimeVo;
import com.usky.system.RemoteMceService;
import com.usky.system.domain.SysUser;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.temporal.TemporalAdjusters;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.stream.Collectors;


/**
 * <p>
 * 工作报告表 服务实现类
 * </p>
 *
 * @author fu
 * @since 2024-05-20
 */
@Slf4j
@Service
public class PmWorkReportServiceImpl extends AbstractCrudService<PmWorkReportMapper, PmWorkReport> implements PmWorkReportService {

    private static final String INFO_TITLE = "报告提醒";
    private static final String INFO_CONTENT = "的报告";
    private static final int INFO_TYPE = 5;

    @Autowired
    private PmWorkContentMapper pmWorkContentMapper;

    @Autowired
    private PmWorkReportMapper pmWorkReportMapper;

    @Autowired
    private PmProjectService pmProjectService;

    @Autowired
    private PmWorkContentService pmWorkContentService;

    @Autowired
    private RemoteMceService remoteMceService;

    @Autowired
    private PmReceiveMapper pmReceiveMapper;

    @Autowired
    private MceMbuserMapper mceMbuserMapper;

    @Autowired
    private SysUserMapper sysUserMapper;

    /**
     * 获取时间内工作报告
     *
     * @param startDate 开始时间
     * @param endDate   结束时间
     * @param reportId  报告id
     * @return
     */
    @Override
    public List<Map<String, List<PmWorkReport>>> weekWork(String startDate, String endDate, Integer reportId) {
        List<Map<String, List<PmWorkReport>>> returnList = new ArrayList<>();
        List<PmWorkReport> report = new ArrayList<>();
        Map<String, List<PmWorkReport>> weekData = new HashMap<>();
        Map<String, List<PmWorkReport>> newData = new HashMap<>();
        LocalDate startDate1 = null;
        LocalDate endDate1 = null;
        if (startDate == null && endDate == null) {
            startDate1 = LocalDate.now().with(TemporalAdjusters.previousOrSame(DayOfWeek.MONDAY));
            endDate1 = LocalDate.now().with(TemporalAdjusters.nextOrSame(DayOfWeek.SUNDAY));
        } else {
            startDate1 = LocalDate.parse(startDate);
            endDate1 = LocalDate.parse(endDate);
        }
        List<LocalDate> dates = new ArrayList<>();
        LocalDate tempDate = startDate1;
        while (!tempDate.isAfter(endDate1)) {
            dates.add(tempDate);
            tempDate = tempDate.plusDays(1);
        }
        //返回最新的一条数据给前端填充
        LambdaQueryWrapper<PmWorkReport> wrapper2 = Wrappers.lambdaQuery();
        wrapper2.eq(PmWorkReport::getSubmitterId, SecurityUtils.getUserId())
                .orderByDesc(PmWorkReport::getSubmitDate)
                .last("LIMIT 1");
        report = baseMapper.selectList(wrapper2);
        if (!report.isEmpty()) {
            Integer pId = report.get(0).getId();
            LambdaQueryWrapper<PmWorkContent> wrapper = Wrappers.lambdaQuery();
            wrapper.eq(PmWorkContent::getReportId, pId);
            List<PmWorkContent> contents1 = pmWorkContentMapper.selectList(wrapper);
            report.get(0).setWorkContents(contents1);
        }
        //固定返回七条数据，没有内容也要设置时间给前端渲染
        LambdaQueryWrapper<PmWorkReport> queryWrapperR = Wrappers.lambdaQuery();
        queryWrapperR.select(PmWorkReport::getId, PmWorkReport::getReportDate, PmWorkReport::getSubmitDate, PmWorkReport::getUpdateTime,
                PmWorkReport::getTomorrowPlan, PmWorkReport::getCoordinateWork, PmWorkReport::getCcTo, PmWorkReport::getSendDingTalk)
                .eq(PmWorkReport::getSubmitterId, SecurityUtils.getUserId())
                .between(PmWorkReport::getReportDate, startDate1, endDate1)
                .orderByAsc(PmWorkReport::getReportDate)
                .apply(reportId != 0, "id = " + reportId);
        List<PmWorkReport> reports = baseMapper.selectList(queryWrapperR);
        if (reports.isEmpty()) {
            List<PmWorkReport> reports1 = new ArrayList<>();
            for (int f = 0; f < dates.size(); f++) {
                List<PmWorkContent> content = new ArrayList<>();
                PmWorkReport report1 = new PmWorkReport();
                report1.setReportDate(dates.get(f));
                report1.setWorkContents(content);
                reports1.add(report1);
            }
            weekData.put("weekData", reports1);
            newData.put("newData", report);
            returnList.add(0, newData);
            returnList.add(1, weekData);
            return returnList;
        }
        List<Integer> ids = new ArrayList<>();
        for (PmWorkReport pwr : reports) {
            ids.add(pwr.getId());
        }
        LambdaQueryWrapper<PmWorkContent> queryWrapperC = Wrappers.lambdaQuery();
        queryWrapperC.select(PmWorkContent::getId, PmWorkContent::getWorkContent, PmWorkContent::getWorkTime, PmWorkContent::getProjectId,
                PmWorkContent::getProjectName, PmWorkContent::getReportId, PmWorkContent::getCreateTime, PmWorkContent::getUpdateTime)
                .in(PmWorkContent::getReportId, ids)
                .orderByDesc(PmWorkContent::getWorkTime);
        List<PmWorkContent> contents = pmWorkContentMapper.selectList(queryWrapperC);
        int a, b;
        for (int i = 0; i < reports.size(); i++) {
            a = reports.get(i).getId();
            List<PmWorkContent> contentList = new ArrayList<>();
            for (int j = 0; j < contents.size(); j++) {
                b = contents.get(j).getReportId();
                if (b == a) {
                    contentList.add(contents.get(j));
                }
            }
            reports.get(i).setWorkContents(contentList);
        }
        if (reports.size() == 7 || reportId != 0) {
            weekData.put("weekData", reports);
            LambdaQueryWrapper<PmWorkReport> wrapper1 = Wrappers.lambdaQuery();
            wrapper1.eq(PmWorkReport::getSubmitterId, SecurityUtils.getUserId())
                    .orderByDesc(PmWorkReport::getReportDate)
                    .last("LIMIT 1");
            report = this.list(wrapper1);
            Integer rId = report.get(0).getId();
            LambdaQueryWrapper<PmWorkContent> wrapperC = Wrappers.lambdaQuery();
            wrapperC.eq(PmWorkContent::getProjectId, rId);
            List<PmWorkContent> pmWorkContents = pmWorkContentMapper.selectList(wrapperC);
            report.get(0).setWorkContents(pmWorkContents);
            newData.put("newData", report);
            returnList.add(0, newData);
            returnList.add(1, weekData);
            return returnList;
        }
        List<PmWorkReport> reportList = new ArrayList<>();
        for (int d = 0; d < dates.size(); d++) {
            boolean matchFound = false;
            for (int c = 0; c < reports.size(); c++) {
                if (dates.get(d).isEqual(reports.get(c).getReportDate())) {
                    reportList.add(reports.get(c));
                    matchFound = true;
                    break;
                }
            }
            if (!matchFound) {
                PmWorkReport newReport = new PmWorkReport();
                List<PmWorkContent> newContent = new ArrayList<>();
                newReport.setReportDate(dates.get(d));
                newReport.setWorkContents(newContent);
                reportList.add(newReport);
            }
        }
        weekData.put("weekData", reportList);
        newData.put("newData", report);
        returnList.add(0, newData);
        returnList.add(1, weekData);
        return returnList;
    }

    /**
     * 新增、编辑工作报告
     */
    @Transactional
    @Override
    public void addReport(PmWorkReport pmWorkReport) {
        Long deptId = SecurityUtils.getLoginUser().getSysUser().getDeptId();
        String userName = SecurityUtils.getUsername();
        Long userId = SecurityUtils.getUserId();
        Integer tenantId = SecurityUtils.getTenantId();
        LocalDateTime dateTime = LocalDateTime.now();
        BigDecimal totalWorkTime = BigDecimal.ZERO; //计算总工时
        if (pmWorkReport.getWorkContents() == null) {
            throw new BusinessException("报告内容不能为空，请检查！");
        }
        Set<Integer> projectIds2 = new HashSet<>();
        for (PmWorkContent workContent : pmWorkReport.getWorkContents()) {
            if (!projectIds2.add(workContent.getProjectId())) {
                throw new BusinessException("存在重复项目，请检查！");
            }
        }
        for (PmWorkContent a : pmWorkReport.getWorkContents()) {
            totalWorkTime = totalWorkTime.add(a.getWorkTime());
        }
        //判断总工时是否超过24h
        BigDecimal maxWorkTimePerDay = BigDecimal.valueOf(24);
        if (totalWorkTime.compareTo(maxWorkTimePerDay) >= 0) {
            throw new BusinessException("一天24小时都不够你用，请检查当日总工时！");
        }
        if (totalWorkTime.compareTo(BigDecimal.ZERO) <= 0) {
            throw new BusinessException("工时必须大于零！");
        }
        LambdaQueryWrapper<PmWorkReport> reportWrapper = Wrappers.lambdaQuery();
        reportWrapper.select(PmWorkReport::getId)
                .eq(PmWorkReport::getReportDate, pmWorkReport.getReportDate())
                .last("LIMIT 1");
        List<PmWorkReport> repeat = this.list(reportWrapper);
        //判断是否携带reportId，不写带则为新增报告，携带则为编辑（更新）项目
        if (pmWorkReport.getId() == null) {

            PmWorkReport newReport = new PmWorkReport();
            newReport.setReportDate(pmWorkReport.getReportDate());
            newReport.setSubmitterId(userId);
            newReport.setSubmitDate(LocalDateTime.now());
            newReport.setCcTo(pmWorkReport.getCcTo());
            newReport.setCoordinateWork(pmWorkReport.getCoordinateWork());
            newReport.setTomorrowPlan(pmWorkReport.getTomorrowPlan());
            newReport.setCreateBy(userName);
            newReport.setCreateTime(dateTime);
            newReport.setDeptId(deptId);
            newReport.setTenantId(tenantId);
            newReport.setTotalHours(totalWorkTime);
            newReport.setSendDingTalk(pmWorkReport.getSendDingTalk());
            try {
                pmWorkReportMapper.insert(newReport);

            } catch (Exception e) {
                e.printStackTrace();
                throw new BusinessException("已存在重复数据，一人一天一篇报告");
            }

            //获取报告中所有项目id
            List<Integer> projectIds = new ArrayList<>();
            for (PmWorkContent b1 : pmWorkReport.getWorkContents()) {
                projectIds.add(b1.getProjectId());
            }
            //查出所有项目id对应项目名
            List<PmProject> project = pmProjectService.projectName(projectIds);
            //将项目名重新赋值
            for (PmWorkContent b : pmWorkReport.getWorkContents()) {
                for (int c = 0; c < project.size(); c++) {
                    if (b.getProjectId() == project.get(c).getId()) {
                        b.setProjectName(project.get(c).getProjectName());
                    }
                }
                PmWorkContent newContent = new PmWorkContent();
                newContent.setReportId(newReport.getId());
                newContent.setProjectId(b.getProjectId());
                projectIds.add(b.getReportId());
                newContent.setProjectName(b.getProjectName());
                newContent.setSubmitterId(userId);
                newContent.setWorkContent(b.getWorkContent());
                newContent.setWorkTime(b.getWorkTime());
                newContent.setCreateBy(userName);
                newContent.setCreateTime(dateTime);
                newContent.setDeptId(deptId);
                newContent.setTenantId(tenantId);
                pmWorkContentMapper.insert(newContent);
            }
            //推送消息中心
            List<Long> ids = new ArrayList<>();
            if (!newReport.getCcTo().isEmpty()) {
                ids = Optional.ofNullable(newReport.getCcTo())
                        .map(ccTo -> Arrays.stream(ccTo.split(","))
                                .map(Long::parseLong)
                                .collect(Collectors.toList()))
                        .orElse(Collections.emptyList());
            }
            sendAsyncMessage(newReport, ids);
            //存入消息接收
            receiveMessages(ids, newReport.getId());
            //是否同步钉钉
            if (pmWorkReport.getSendDingTalk() == 1) {
                sendDingTalkDailyReport(pmWorkReport);
            }
        } else if (repeat.size() > 0) {
            PmWorkReport rp = new PmWorkReport();
            rp.setId(pmWorkReport.getId());
            rp.setReportDate(pmWorkReport.getReportDate());
            rp.setSubmitterId(pmWorkReport.getSubmitterId());
            rp.setSubmitDate(pmWorkReport.getSubmitDate());
            rp.setCcTo(pmWorkReport.getCcTo());
            rp.setCoordinateWork(pmWorkReport.getCoordinateWork());
            rp.setTomorrowPlan(pmWorkReport.getTomorrowPlan());
            rp.setCreateBy(pmWorkReport.getCreateBy());
            rp.setCreateTime(pmWorkReport.getCreateTime());
            rp.setUpdateTime(dateTime);
            rp.setDeptId(pmWorkReport.getDeptId());
            rp.setTenantId(pmWorkReport.getTenantId());
            rp.setUpdateBy(userName);
            rp.setUpdateTime(dateTime);
            rp.setTotalHours(totalWorkTime);
            pmWorkReportMapper.updateById(rp);
            LambdaQueryWrapper<PmWorkContent> queryWrapper = Wrappers.lambdaQuery();
            queryWrapper.select(PmWorkContent::getReportId, PmWorkContent::getSubmitterId, PmWorkContent::getCreateBy, PmWorkContent::getCreateTime, PmWorkContent::getDeptId, PmWorkContent::getTenantId)
                    .eq(PmWorkContent::getReportId, pmWorkReport.getId())
                    .last("LIMIT 1");
            PmWorkContent f = pmWorkContentMapper.selectOne(queryWrapper);
            pmWorkContentService.deleteContent(pmWorkReport.getId());
            List<PmWorkContent> contents = pmWorkReport.getWorkContents();
            for (PmWorkContent e : contents) {
                e.setReportId(f.getReportId());
                e.setUpdateBy(userName);
                e.setUpdateTime(dateTime);
                e.setSubmitterId(f.getSubmitterId());
                e.setCreateTime(f.getCreateTime());
                e.setCreateBy(f.getCreateBy());
                e.setDeptId(f.getDeptId());
                e.setTenantId(f.getTenantId());
                pmWorkContentMapper.insert(e);
            }
        }
    }

    private void receiveMessages(List<Long> ids, Integer reportId) {
        List<SysUser> users = pmWorkContentService.nickNames(ids);
        if (ids.size() > 0) {
            for (Long id : ids) {
                PmReceive pmReceive = new PmReceive();
                pmReceive.setReceiverId(id);
                pmReceive.setReportId(reportId);
                pmReceive.setTenantId(SecurityUtils.getTenantId());
                pmReceive.setDeptId(SecurityUtils.getLoginUser().getSysUser().getDeptId());
                pmReceive.setCreateBy(SecurityUtils.getUsername());
                pmReceive.setCreateTime(LocalDateTime.now());
                for (SysUser user : users) {
                    if (user.getUserId().equals(id)) {
                        pmReceive.setReceiverName(user.getUserName());
                    }
                }
                pmReceive.setReadFlag(0);
                pmReceiveMapper.insert(pmReceive);
            }
        }
    }

    @Async("asyncServiceExecutor")
    public void sendAsyncMessage(PmWorkReport newReport, List<Long> userId) {
        log.info("start asyncServiceExecutor remoteMceService.addMce--------------------");
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("infoTitle", INFO_TITLE);
        jsonObject.put("infoContent", SecurityUtils.getLoginUser().getSysUser().getNickName() + INFO_CONTENT);
        jsonObject.put("infoType", INFO_TYPE);
        jsonObject.put("id", newReport.getId());
        jsonObject.put("infoTypeName", INFO_TITLE);
        if (userId != null && !userId.isEmpty()) {
            jsonObject.put("userIds", userId);
        }
        // 推送消息中心
        try {
            remoteMceService.addMce(jsonObject.toString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public List<Map<String, Object>> countTime() {
        int scale = 2;
        RoundingMode roundingMode = RoundingMode.HALF_UP;
        List<Map<String, Object>> returnList = new ArrayList<>();
        Map<String, Object> weekTotalMap = new HashMap<>();
        Map<String, Object> monthTotalMap = new HashMap<>();
        Map<String, Object> weekMap = new HashMap<>();
        Map<String, Object> monthMap = new HashMap<>();
        List<PmProjectWorkTimeVo> returnListW = new ArrayList<>();
        List<PmProjectWorkTimeVo> returnListM = new ArrayList<>();
        LocalDate today = LocalDate.now();
        LocalDate startOfWeek = today.with(TemporalAdjusters.previousOrSame(java.time.DayOfWeek.MONDAY));
        LocalDate endOfWeek = today.with(TemporalAdjusters.nextOrSame(java.time.DayOfWeek.SUNDAY));
        LocalDateTime startOfWeekDateTime = startOfWeek.atStartOfDay();
        LocalDateTime endOfWeekDateTime = endOfWeek.atTime(23, 59, 59);
        LocalDate startOfMonth = today.withDayOfMonth(1);
        LocalDate endOfMonth = today.withDayOfMonth(today.lengthOfMonth());
        LocalDateTime startOfMonthDateTime = startOfMonth.atStartOfDay();
        LocalDateTime endOfMonthDateTime = endOfMonth.atTime(23, 59, 59);

        BigDecimal weekTotal = BigDecimal.ZERO;
        List<PmProjectWorkTimeVo> week = pmWorkContentMapper.workTimeCount(startOfWeekDateTime, endOfWeekDateTime, SecurityUtils.getUserId());
        if (week.isEmpty()) {
            weekTotalMap.put("WeekWorkTimeTotal", 0);
            weekMap.put("WeeklyData", week);
        }
        for (int i = 0; i < week.size(); i++) {
            weekTotal = weekTotal.add(week.get(i).getWorkTime());
        }
        BigDecimal totalPercentageW = BigDecimal.ZERO;
        for (int a = 0; a < week.size(); a++) {
            PmProjectWorkTimeVo workTimeVo = new PmProjectWorkTimeVo();
            workTimeVo.setProjectName(week.get(a).getProjectName());
            workTimeVo.setWorkTime(week.get(a).getWorkTime());
            if (a != week.size() - 1) {
                BigDecimal percentage = week.get(a).getWorkTime().divide(weekTotal, scale, roundingMode);
                workTimeVo.setPercentage(percentage);
                totalPercentageW = totalPercentageW.add(percentage);
            } else {
                BigDecimal lastPercentage = BigDecimal.ONE.subtract(totalPercentageW);
                workTimeVo.setPercentage(lastPercentage);
            }
            workTimeVo.setPercentage(week.get(a).getWorkTime().divide(weekTotal, scale, roundingMode));
            returnListW.add(workTimeVo);
        }

        BigDecimal monthTotal = BigDecimal.ZERO;
        List<PmProjectWorkTimeVo> month = pmWorkContentMapper.workTimeCount(startOfMonthDateTime, endOfMonthDateTime, SecurityUtils.getUserId());
        if (month.isEmpty()) {
            weekTotalMap.put("MonthWorkTimeTotal", 0);
            weekMap.put("MonthlyData", month);
        }
        for (int i = 0; i < month.size(); i++) {
            monthTotal = monthTotal.add(month.get(i).getWorkTime());
        }
        BigDecimal totalPercentageM = BigDecimal.ZERO;
        for (int b = 0; b < month.size(); b++) {
            PmProjectWorkTimeVo workTimeVo = new PmProjectWorkTimeVo();
            workTimeVo.setProjectName(month.get(b).getProjectName());
            workTimeVo.setWorkTime(month.get(b).getWorkTime());
            if (b != month.size() - 1) {
                BigDecimal percentage = month.get(b).getWorkTime().divide(monthTotal, scale, roundingMode);
                workTimeVo.setPercentage(percentage);
                totalPercentageM = totalPercentageM.add(percentage);
            } else {
                BigDecimal lastPercentage = BigDecimal.ONE.subtract(totalPercentageM);
                workTimeVo.setPercentage(lastPercentage);
            }

            returnListM.add(workTimeVo);
        }

        weekTotalMap.put("WeekWorkTimeTotal", weekTotal);
        monthTotalMap.put("MonthWorkTimeTotal", monthTotal);
        weekMap.put("WeeklyData", returnListW);
        monthMap.put("MonthlyData", returnListM);
        returnList.add(weekTotalMap);
        returnList.add(monthTotalMap);
        returnList.add(weekMap);
        returnList.add(monthMap);
        return returnList;
    }

    /**
     * 指定请求
     * @return
     * @throws Exception
     */
    public static com.aliyun.dingtalkoauth2_1_0.Client createClient() throws Exception {
        com.aliyun.teaopenapi.models.Config config = new com.aliyun.teaopenapi.models.Config();
        config.protocol = "https";
        config.regionId = "central";
        config.method = "POST";
        return new com.aliyun.dingtalkoauth2_1_0.Client(config);
    }

    /**
     * 获取钉钉企业内部AccessToken
     * @return
     * @throws Exception
     */
    public String getDingTalkToken() throws Exception {
        GetAccessTokenResponse responseBody = new GetAccessTokenResponse();
        com.aliyun.dingtalkoauth2_1_0.Client client = createClient();
        com.aliyun.dingtalkoauth2_1_0.models.GetAccessTokenRequest getAccessTokenRequest = new com.aliyun.dingtalkoauth2_1_0.models.GetAccessTokenRequest()
                .setAppKey(dingTalkConstant.DING_TALK_CLIENT_ID)
                .setAppSecret(dingTalkConstant.DING_TALK_CLIENT_SECRET);
        try {
            responseBody = client.getAccessToken(getAccessTokenRequest);
        } catch (TeaException err) {
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                // err 中含有 code 和 message 属性，可帮助开发定位问题
            }
        } catch (Exception _err) {
            TeaException err = new TeaException(_err.getMessage(), _err);
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                // err 中含有 code 和 message 属性，可帮助开发定位问题
            }
        }
        return responseBody.getBody().accessToken;
    }

    /**
     * 获取钉钉userId
     * @param userId 平台userid
     * @return
     */
    public String getDingTalkUserId(Long userId) {
        String dingTalkId = "";
        LambdaQueryWrapper<MceMbuser> dingTalkQuery = Wrappers.lambdaQuery();
        dingTalkQuery.select(MceMbuser::getDingTalkId, MceMbuser::getUserId)
                .eq(MceMbuser::getUserId, userId);
        MceMbuser mceMbuser = mceMbuserMapper.selectOne(dingTalkQuery);
        if (mceMbuser != null) {
            String dingId = mceMbuser.getDingTalkId();
            if (dingId != null && dingId != "") {
                return mceMbuser.getDingTalkId();
            }
        }
            LambdaQueryWrapper<SysUser> phoneQuery = Wrappers.lambdaQuery();
            phoneQuery.select(SysUser::getPhonenumber, SysUser::getUserName)
                    .eq(SysUser::getUserId, userId);
            SysUser user = sysUserMapper.selectOne(phoneQuery);
            String mobile = user.getPhonenumber();
            try {
                String accessToken = getDingTalkToken();
                DingTalkClient client = new DefaultDingTalkClient(dingTalkConstant.DING_TALK_USERID_URL);
                OapiV2UserGetbymobileRequest req = new OapiV2UserGetbymobileRequest();
                req.setMobile(mobile);
                OapiV2UserGetbymobileResponse rsp = client.execute(req, accessToken);
                ObjectMapper objectMapper = new ObjectMapper();
                JsonNode rootNode = objectMapper.readTree(rsp.getBody());
                JsonNode resultNode = rootNode.get("result");
                dingTalkId = resultNode.get("userid").asText();
            } catch (Exception e) {
                e.printStackTrace();
            }
            if (mceMbuser != null) {
                mceMbuser.setDingTalkId(dingTalkId);
                mceMbuser.setUpdateBy(SecurityUtils.getUsername());
                mceMbuser.setUpdateTime(LocalDateTime.now());
                mceMbuserMapper.update(mceMbuser, dingTalkQuery);
            } else {
                MceMbuser mbuser = new MceMbuser();
                mbuser.setDingTalkId(dingTalkId);
                mbuser.setPhone(mobile);
                mbuser.setUserId(userId);
                mbuser.setCreateBy(user.getUserName());
                mbuser.setCreateTime(LocalDateTime.now());
                mceMbuserMapper.insert(mbuser);
            }
        return dingTalkId;
    }

    @Async("asyncServiceExecutor")//异步发送
    @Override
    public void sendDingTalkDailyReport(PmWorkReport workReport) {
        String ccToStr = workReport.getCcTo();
        String[] items = ccToStr.split(",");
        List<Long> userIds = new ArrayList<>();
        for (String item : items) {
            long number = Long.parseLong(item.trim());
            userIds.add(number);
        }
        List<String> ccTo = new ArrayList<>();
        for (Long userId : userIds) {
            String dingTalkUserId = getDingTalkUserId(userId);
            ccTo.add(dingTalkUserId);
        }
        String coordinateWork1 = workReport.getCoordinateWork();
        String tomorrowPlan1 = workReport.getTomorrowPlan();
        List<PmWorkContent> workContents = workReport.getWorkContents();
        StringBuilder contentBuilder = new StringBuilder();
        for (PmWorkContent content : workContents) {
            String projectName = content.getProjectName();
            BigDecimal workTime = content.getWorkTime();
            String workContent = content.getWorkContent();
            StringBuilder markdown = new StringBuilder();
            markdown.append("## ").append(projectName).append(" ").append(workTime).append("h\n\n");
            markdown.append(workContent).append("\n");
            contentBuilder.append(markdown);
        }
        String completedWork = contentBuilder.toString();
        try {
            DingTalkClient client = new DefaultDingTalkClient(dingTalkConstant.DING_TALK_CREATE_DAILY_REPORT_URL);
            OapiReportCreateRequest req = new OapiReportCreateRequest();
            OapiReportCreateRequest.OapiCreateReportParam obj1 = new OapiReportCreateRequest.OapiCreateReportParam();
            List<OapiReportCreateRequest.OapiReportContentVo> list3 = new ArrayList<>();
            OapiReportCreateRequest.OapiReportContentVo completedWorkD = new OapiReportCreateRequest.OapiReportContentVo();
            completedWorkD.setSort(0L);
            completedWorkD.setType(1L);
            completedWorkD.setContentType(dingTalkConstant.DING_TALK_DAILY_REPORT_CONTENT_TYPE);
            completedWorkD.setContent(completedWork);
            completedWorkD.setKey(dingTalkConstant.DING_TALK_DAILY_REPORT_COMPLETED_WORK);
            list3.add(completedWorkD);
            OapiReportCreateRequest.OapiReportContentVo tomorrowPlan = new OapiReportCreateRequest.OapiReportContentVo();
            tomorrowPlan.setSort(1L);
            tomorrowPlan.setType(1L);
            tomorrowPlan.setContentType(dingTalkConstant.DING_TALK_DAILY_REPORT_CONTENT_TYPE);
            tomorrowPlan.setContent(tomorrowPlan1);
            tomorrowPlan.setKey(dingTalkConstant.DING_TALK_DAILY_REPORT_TOMORROW_PLAN);
            list3.add(tomorrowPlan);
            OapiReportCreateRequest.OapiReportContentVo coordinateWork = new OapiReportCreateRequest.OapiReportContentVo();
            coordinateWork.setSort(2L);
            coordinateWork.setType(1L);
            coordinateWork.setContentType(dingTalkConstant.DING_TALK_DAILY_REPORT_CONTENT_TYPE);
            coordinateWork.setContent(coordinateWork1);
            coordinateWork.setKey(dingTalkConstant.DING_TALK_DAILY_REPORT_COORDINATE_WORK);
            list3.add(coordinateWork);
            obj1.setContents(list3);
            obj1.setToUserids(ccTo);
            obj1.setTemplateId(dingTalkConstant.DING_TALK_DAILY_REPORT_TEMPLATE_ID);
            obj1.setToChat(dingTalkConstant.DING_TALK_DAILY_REPORT_TO_CHAT);
            obj1.setDdFrom(dingTalkConstant.DING_TALK_CORP_ID);
            obj1.setUserid(getDingTalkUserId(SecurityUtils.getUserId()));
            //obj1.setToCids(dingTalkConstant.DING_TALK_DAILY_REPORT_TO_CIDS);//发送到群，群id
            req.setCreateReportParam(obj1);
            OapiReportCreateResponse rsp = client.execute(req, getDingTalkToken());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


}
